import sys
import iotbx.pdb
from cctbx import maptbx
from cctbx import miller
from mmtbx import map_tools
import mmtbx.utils
from libtbx.utils import null_out
from mmtbx.maps import fem

def get_map(mc, cg):
  fft_map = miller.fft_map(
      crystal_gridding     = cg,
      fourier_coefficients = mc)
  fft_map.apply_sigma_scaling()
  return fft_map.real_map_unpadded()
    
def run(args):
  inputs = mmtbx.utils.process_command_line_args(args = args)
  df = mmtbx.utils.determine_data_and_flags(
    reflection_file_server  = inputs.get_reflection_file_server(),
    log = null_out())
  pdb_inp = iotbx.pdb.input(file_name=inputs.pdb_file_names[0])
  xrs = pdb_inp.xray_structure_simple()
  
  f_obs = df.f_obs.average_bijvoet_mates()
  f_obs, r_free_flags = f_obs.common_sets(df.r_free_flags)
  
  fmodel = mmtbx.f_model.manager(
    f_obs          = f_obs,
    r_free_flags   = r_free_flags,
    xray_structure = xrs)
  fmodel.update_all_scales()
  print fmodel.r_work(), fmodel.r_free()
  #
  mc_orig = mmtbx.map_tools.electron_density_map(fmodel=fmodel).map_coefficients(
    map_type     = "2mFo-DFc",
    isotropize   = False,
    fill_missing = False)
  crystal_gridding = f_obs.crystal_gridding(
    d_min             = fmodel.f_obs().d_min(),
    symmetry_flags    = maptbx.use_space_group_symmetry,
    resolution_factor = 1./4)
  #
  m_orig = get_map(mc=mc_orig, cg=crystal_gridding)
  m_he  = maptbx.volume_scale(map = m_orig,  n_bins = 10000).map_data()
  #
  fem.ccp4_map(cg=crystal_gridding, file_name="orig.ccp4", map_data=m_orig)
  fem.ccp4_map(cg=crystal_gridding, file_name="he.ccp4", map_data=m_he)
  

if (__name__ == "__main__"):
  run(sys.argv[1:])

